from .ApiData import A

__url__ = 'https://data.shuvtai.cn/'


class URL(A):
    def __init__(self, _url):
        A.__init__(
            self,
            {
                'chat': _url + '/chat',
                'new': _url + '/new',
                'apply': _url + '/apply',
            }
        )
        self.chat = _url + '/chat'
        self.new = _url + '/new'
        self.apply = _url + '/apply'



# *----------------------------------------------------------------
# * MobAI.API.ApiData
# * Write by MoYeRanQianZhi
# * 2023.7.26
# *----------------------------------------------------------------

import json


class A:
    def __init__(self, _data):
        self._data = _data

    def __str__(self):
        return json.dumps(self._data, indent=4)

    def __repr__(self):
        return json.dumps(self._data, indent=4)


class ChatData(A):
    def __init__(self, _data):
        A.__init__(self, _data)
        self.time = _data['time']
        self.state = _data['state']
        if _data['state'] == 'success':
            self.data = A(_data['data'])
            self.data.reply = _data['data']['reply']
        else:
            self.data = A(_data['data'])
            self.data.error = _data['data']['error']


class NewData(A):
    def __init__(self, _data):
        A.__init__(self, _data)
        self.time = _data['time']
        self.state = _data['state']
        if _data['state'] == 'success':
            self.data = A(_data['data'])
            self.data.id = _data['data']['id']
            self.data.password = _data['data']['password']
            self.data.reply = _data['data']['reply']
        else:
            self.data = A(_data['data'])
            self.data.error = _data['data']['error']



# *----------------------------------------------------------------
# * MobAI.API.api
# * Write by MoYeRanQianZhi
# * 2023.7.26
# *----------------------------------------------------------------

import requests
from .ApiData import ChatData, NewData
from . import common
from .error import ApiKeyType

_api_key = None
url = common.URL(
    requests.get(common.__url__).text
)


class ChatAPI(ChatData):
    def __init__(self, _id, password, question, api_key=None):
        self.id = _id
        self.password = password
        self.question = question

        if api_key is not None:
            self.api_key = api_key
        elif _api_key is not None:
            self.api_key = _api_key
        else:
            raise ApiKeyType('None')

        ChatData.__init__(self, self.ask())

    def ask(self):
        return requests.post(
            url=url.chat,
            json={
                'key': self.api_key,
                'id': self.id,
                'password': self.password,
                'question': self.question,
            }
        ).json()

    def dict(self):
        return self._data


class CreateNewAPI(NewData):
    def __init__(self, model, api_key=None):
        self.model = model

        if api_key is not None:
            self.api_key = api_key
        elif _api_key is not None:
            self.api_key = _api_key
        else:
            raise ApiKeyType('None')

        NewData.__init__(self, self.create())

    def create(self):
        return requests.post(
            url=url.new,
            json={
                'key': self.api_key,
                'model': self.model,
            }
        ).json()

    def dict(self):
        return self._data


def apply(email):
    return requests.post(
        url.apply,
        json={'mail': email}
    )



from setuptools import setup, find_packages

LICENSE = '''MIT License

Copyright (c) 2023 MYR

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
'''

md = '''# ShanHeUniversity

## **使用方法**

### **使用流程总览**

1.申请密钥(apply)

2.新建记录(CreateNewAPI)

3.循环聊天(ChatAPI)

---

### **申请密钥**

#### **准备工作**
1.Python3.*

2.`pip install shanheuniversity`

#### **示例代码**

```python
from ShanHeuniversity import api

api.apply(email='demo@shu.edu')

```

#### **参数总览**

| 参数名 | 参数类型 | 是否必填 | 默认值 | 示例 |
| --- | --- | --- | --- | --- |
| email | String | 是 | 无 | sb@xxx.com |

---

### **新建记录**

#### **准备工作**
1.Python3.*

2.`pip install shanheuniversity`

#### **示例代码**

```python
from ShanHeuniversity import api
import sys
import os

function = lambda data: sys.stdout.write(str(data))

api._api_key = os.getenv('MOBAI_API_KEY')
function(
    api.CreateNewAPI(model='MobAI')
)

```

#### **参数总览**

| 参数名 | 参数类型 | 是否必填 | 默认值 | 示例 |
| --- | --- | --- | --- | --- |
| api_key | String | 是 | 无 | sk-demo |
| model | String | 是 | 无 | 天马行空 |

---

### **循环聊天**

#### **准备工作**
1.Python3.*

2.`pip install shanheuniversity`

#### **示例代码**

```python
import sys
import os
from ShanHeuniversity import api

function = lambda data: sys.stdout.write(
    'MobAI:' + data.data.reply + '\n\n'
    if data.state == 'success'
    else 'ERROR:' + data.data.error + '\n\n'
)
ask = input

api._api_key = os.getenv('MOBAI_API_KEY')
while True:
    function(
        api.ChatAPI(
            _id='id',
            password='password',
            question=ask('User:')
        )
    )

```

#### **参数总览**

| 参数名 | 参数类型 | 是否必填 | 默认值 | 示例 |
| --- | --- | --- | --- | --- |
| api_key | String | 是 | 无 | sk-demo |
| _id | String | 是 | 无 | 0000-0000-0000-0000 |
| password | String | 是 | 无 | password-1234 |
| question | String | 是 | 无 | Hello! |


### 可参考示例

> 以下代码可直接运行，但不建议，因为这只是一个最基本的示例。

```python
import sys
from ShanHeuniversity import api

ask = input
function = lambda data: sys.stdout.write(
    'MobAI:' + str(data.get('data').get('reply')) + '\n\n'
    if data.get('state') == 'success'
    else 'ERROR:' + data.get('data').get('error') + '\n\n'
)

api.apply(email=ask('Mail:'))

api._api_key = ask('Key:')

ai = api.CreateNewAPI(model=ask('Model:'))

while True:
    function(
        api.ChatAPI(
            _id=ai.data.id,
            password=ai.data.password,
            question=ask('User:')
        )
    )

```
'''

setup(
    name='ShanHeuniversity',
    version='0.1.0',
    packages=find_packages(),
    url='https://shu.university/',
    license=LICENSE,
    author='MoYeRanQianZhi',
    author_email='moyeranqianzhi@gmail.com',
    description='This is a module for MobAI.API.',
    long_description=md,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    python_requires='>=3.6.0',
    requires=[
        'requests'
    ]
)



class ApiKeyType(Exception):
    def __init__(self, _type):
        self.value = f'The API key is {_type}, should be str.'

    def __str__(self):
        return repr(self.value)

    __module__ = 'builtins'
